@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,17 +29,35 @@
 
  */
 
-/* $Id: apc_globals.h,v 3.70.2.8 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_globals.h 284222 2009-07-17 01:23:57Z shire $ */
 
 #ifndef APC_GLOBALS_H
 #define APC_GLOBALS_H
 
-#define APC_VERSION "3.0.19"
-
 #include "apc_cache.h"
 #include "apc_stack.h"
 #include "apc_php.h"
 
+/* {{{ struct apc_rfc1867_data */
+
+typedef struct _apc_rfc1867_data apc_rfc1867_data;
+
+struct _apc_rfc1867_data {
+    char tracking_key[64];
+    int  key_length;
+    size_t content_length;
+    char filename[128];
+    char name[64];
+    char *temp_filename;
+    int cancel_upload;
+    double start_time;
+    size_t bytes_processed;
+    size_t prev_bytes_processed;
+    int update_freq;
+    double rate;
+};
+/* }}} */
+
 ZEND_BEGIN_MODULE_GLOBALS(apc)
     /* configuration parameters */
     zend_bool enabled;      /* if true, apc is enabled (defaults to true) */
@@ -54,40 +72,49 @@ ZEND_BEGIN_MODULE_GLOBALS(apc)
     char *mmap_file_mask;   /* mktemp-style file-mask to pass to mmap */
 #endif
     char** filters;         /* array of regex filters that prevent caching */
+    void* compiled_filters; /* compiled regex filters */
 
     /* module variables */
     zend_bool initialized;       /* true if module was initialized */
     apc_stack_t* cache_stack;    /* the stack of cached executable code */
     zend_bool cache_by_default;  /* true if files should be cached unless filtered out */
                                  /* false if files should only be cached if filtered in */
-    long slam_defense;           /* Probability of a process not caching an uncached file */
-    size_t* mem_size_ptr;        /* size of blocks allocated to file being cached (NULL outside my_compile_file) */
     long file_update_protection; /* Age in seconds before a file is eligible to be cached - 0 to disable */
     zend_bool enable_cli;        /* Flag to override turning APC off for CLI */
-    long max_file_size;	         /* Maximum size of file, in bytes that APC will be allowed to cache */
-    long slam_rand;              /* A place to store the slam rand value for the request */
+    long max_file_size;          /* Maximum size of file, in bytes that APC will be allowed to cache */
     zend_bool fpstat;            /* true if fullpath includes should be stat'ed */
+    zend_bool canonicalize;      /* true if relative paths should be canonicalized in no-stat mode */
     zend_bool stat_ctime;        /* true if ctime in addition to mtime should be checked */
     zend_bool write_lock;        /* true for a global write lock */
     zend_bool report_autofilter; /* true for auto-filter warnings */
-    zend_bool include_once;	     /* Override the ZEND_INCLUDE_OR_EVAL opcode handler to avoid pointless fopen()s [still experimental] */
+    zend_bool include_once;      /* Override the ZEND_INCLUDE_OR_EVAL opcode handler to avoid pointless fopen()s [still experimental] */
     apc_optimize_function_t apc_optimize_function;   /* optimizer function callback */
 #ifdef MULTIPART_EVENT_FORMDATA
-    zend_bool rfc1867;           /* Flag to enable rfc1867 handler */
-    char* rfc1867_prefix;        /* Key prefix */
-    char* rfc1867_name;          /* Name of hidden field to activate upload progress/key suffix */
-    double rfc1867_freq;         /* Update frequency as percentage or bytes */
-#endif
-    HashTable *copied_zvals;     /* my_copy recursion detection list */
-#ifdef ZEND_ENGINE_2
-    int reserved_offset;         /* offset for apc info in op_array->reserved[] */
+    zend_bool rfc1867;            /* Flag to enable rfc1867 handler */
+    char* rfc1867_prefix;         /* Key prefix */
+    char* rfc1867_name;           /* Name of hidden field to activate upload progress/key suffix */
+    double rfc1867_freq;          /* Update frequency as percentage or bytes */
+    long rfc1867_ttl;             /* TTL for rfc1867 entries */
+    apc_rfc1867_data rfc1867_data;/* Per-request data */
 #endif
+    HashTable copied_zvals;      /* my_copy recursion detection list */
     zend_bool force_file_update; /* force files to be updated during apc_compile_file */
     char canon_path[MAXPATHLEN]; /* canonical path for key data */
 #if APC_FILEHITS
-    zval *filehits;             /* Files that came from the cache for this request */
+    zval *filehits;              /* Files that came from the cache for this request */
 #endif
     zend_bool coredump_unmap;    /* Trap signals that coredump and unmap shared memory */
+    apc_cache_t *current_cache;  /* current cache being modified/read */
+    char *preload_path;
+    zend_bool file_md5;           /* record md5 hash of files */
+    void *apc_bd_alloc_ptr;      /* bindump alloc() ptr */
+    void *apc_bd_alloc_ubptr;    /* bindump alloc() upper bound ptr */
+    HashTable apc_bd_alloc_list; /* bindump alloc() ptr list */
+    zend_bool use_request_time;  /* use the SAPI request start time for TTL */
+    zend_bool lazy_functions;        /* enable/disable lazy function loading */
+    HashTable *lazy_function_table;  /* lazy function entry table */
+    zend_bool lazy_classes;          /* enable/disable lazy class loading */
+    HashTable *lazy_class_table;     /* lazy class entry table */
 ZEND_END_MODULE_GLOBALS(apc)
 
 /* (the following declaration is defined in php_apc.c) */
@@ -111,6 +138,6 @@ extern void* apc_compiled_filters;   /* compiled filters */
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
