@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_lock.h,v 3.20.2.1 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_lock.h 268620 2008-11-09 02:30:49Z shire $ */
 
 #ifndef APC_LOCK
 #define APC_LOCK
@@ -34,31 +34,32 @@
 #include "apc_sem.h"
 #include "apc_fcntl.h"
 #include "apc_pthreadmutex.h"
-#include "apc_futex.h"
 #include "apc_spin.h"
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef TSRM_LOCKS
+/* {{{ generic locking macros */
+#define CREATE_LOCK(lock)     apc_lck_create(NULL, 0, 1, lock)
+#define DESTROY_LOCK(lock)    apc_lck_destroy(lock)
+#define LOCK(lock)          { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_lock(lock); }
+#define RDLOCK(lock)        { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_rdlock(lock); }
+#define UNLOCK(lock)        { apc_lck_unlock(lock); HANDLE_UNBLOCK_INTERRUPTIONS(); }
+/* }}} */
+
+#if defined(APC_SEM_LOCKS)
+#define APC_LOCK_TYPE "IPC Semaphore"
 #define RDLOCK_AVAILABLE 0
-#define NONBLOCKING_LOCK_AVAILABLE 0
-/* quick & dirty: use TSRM mutex locks for now */
-#define apc_lck_create(a,b,c,d) d=(int)tsrm_mutex_alloc()
-#define apc_lck_destroy(a)    tsrm_mutex_free((MUTEX_T)a)
-#define apc_lck_lock(a)       tsrm_mutex_lock((MUTEX_T)a)
-#define apc_lck_rdlock(a)     tsrm_mutex_lock((MUTEX_T)a)
-#define apc_lck_unlock(a)     tsrm_mutex_unlock((MUTEX_T)a)
-#elif defined(APC_SEM_LOCKS)
-#define RDLOCK_AVAILABLE 0
-#define NONBLOCKING_LOCK_AVAILABLE 0
+#define NONBLOCKING_LOCK_AVAILABLE 1 
 #define apc_lck_t int
 #define apc_lck_create(a,b,c,d) d=apc_sem_create(NULL,(b),(c))
 #define apc_lck_destroy(a)    apc_sem_destroy(a)
 #define apc_lck_lock(a)       apc_sem_lock(a)
+#define apc_lck_nb_lock(a)    apc_sem_nonblocking_lock(a)
 #define apc_lck_rdlock(a)     apc_sem_lock(a)
 #define apc_lck_unlock(a)     apc_sem_unlock(a)
 #elif defined(APC_PTHREADMUTEX_LOCKS)
+#define APC_LOCK_TYPE "pthread mutex Locks"
 #define RDLOCK_AVAILABLE 0
 #define NONBLOCKING_LOCK_AVAILABLE 1
 #define apc_lck_t pthread_mutex_t 
@@ -68,16 +69,9 @@
 #define apc_lck_nb_lock(a)    apc_pthreadmutex_nonblocking_lock(&a)
 #define apc_lck_rdlock(a)     apc_pthreadmutex_lock(&a)
 #define apc_lck_unlock(a)     apc_pthreadmutex_unlock(&a)
-#elif defined(APC_FUTEX_LOCKS)
-#define NONBLOCKING_LOCK_AVAILABLE 1 
-#define apc_lck_t int 
-#define apc_lck_create(a,b,c,d) d=apc_futex_create()
-#define apc_lck_destroy(a)    apc_futex_destroy(&a)
-#define apc_lck_lock(a)       apc_futex_lock(&a)
-#define apc_lck_nb_lock(a)    apc_futex_nonblocking_lock(&a)
-#define apc_lck_rdlock(a)     apc_futex_lock(&a)
-#define apc_lck_unlock(a)     apc_futex_unlock(&a)
 #elif defined(APC_SPIN_LOCKS)
+#define APC_LOCK_TYPE "spin Locks"
+#define RDLOCK_AVAILABLE 0
 #define NONBLOCKING_LOCK_AVAILABLE APC_SLOCK_NONBLOCKING_LOCK_AVAILABLE
 #define apc_lck_t slock_t 
 #define apc_lck_create(a,b,c,d) apc_slock_create((slock_t*)&(d))
@@ -87,6 +81,7 @@
 #define apc_lck_rdlock(a)     apc_slock_lock(&a)
 #define apc_lck_unlock(a)     apc_slock_unlock(&a)
 #else
+#define APC_LOCK_TYPE "File Locks"
 #define RDLOCK_AVAILABLE 1
 #ifdef PHP_WIN32
 #define NONBLOCKING_LOCK_AVAILABLE 0
