@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,9 +25,11 @@
 
  */
 
-/* $Id: apc_mmap.c,v 3.7.2.2 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_mmap.c 287248 2009-08-13 15:13:52Z gopalv $ */
 
 #include "apc.h"
+#include "apc_mmap.h"
+#include "apc_lock.h"
 
 #if APC_MMAP
 
@@ -35,7 +37,7 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 
-/* 
+/*
  * Some operating systems (like FreeBSD) have a MAP_NOSYNC flag that
  * tells whatever update daemons might be running to not flush dirty
  * vm pages to disk unless absolutely necessary.  My guess is that
@@ -46,17 +48,37 @@
 #define MAP_NOSYNC 0
 #endif
 
-void *apc_mmap(char *file_mask, size_t size)
+/* support for systems where MAP_ANONYMOUS is defined but not MAP_ANON, ie: HP-UX bug #14615 */
+#if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
+# define MAP_ANON MAP_ANONYMOUS
+#endif
+
+apc_segment_t apc_mmap(char *file_mask, size_t size)
 {
-    void* shmaddr;  /* the shared memory address */
+    apc_segment_t segment; 
+
+    int fd = -1;
+    int flags = MAP_SHARED | MAP_NOSYNC;
+    int remap = 1;
 
     /* If no filename was provided, do an anonymous mmap */
     if(!file_mask || (file_mask && !strlen(file_mask))) {
-        shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
-    } else {
-        int fd;
-
-        /* 
+#if !defined(MAP_ANON)
+        apc_eprint("Anonymous mmap does not apear to be available on this system (MAP_ANON/MAP_ANONYMOUS).  Please see the apc.mmap_file_mask INI option.");
+#else
+        fd = -1;
+        flags = MAP_SHARED | MAP_ANON;
+        remap = 0;
+#endif
+    } else if(!strcmp(file_mask,"/dev/zero")) { 
+        fd = open("/dev/zero", O_RDWR, S_IRUSR | S_IWUSR);
+        if(fd == -1) {
+            apc_eprint("apc_mmap: open on /dev/zero failed:");
+            goto error;
+        }
+        remap = 0; /* cannot remap */
+    } else if(strstr(file_mask,".shm")) {
+        /*
          * If the filemask contains .shm we try to do a POSIX-compliant shared memory
          * backed mmap which should avoid synchs on some platforms.  At least on
          * FreeBSD this implies MAP_NOSYNC and on Linux it is equivalent of mmap'ing
@@ -66,65 +88,79 @@ void *apc_mmap(char *file_mask, size_t size)
          * On FreeBSD these are mapped onto the regular filesystem so you can put whatever
          * path you want here.
          */
-        if(strstr(file_mask,".shm")) {
-            mktemp(file_mask);
-            fd = shm_open(file_mask, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
-            if(fd == -1) {
-                apc_eprint("apc_mmap: shm_open on %s failed:", file_mask);
-                return (void *)-1;
-            }
-            if (ftruncate(fd, size) < 0) {
-                close(fd);
-                shm_unlink(file_mask);
-                apc_eprint("apc_mmap: ftruncate failed:");
-                return (void *)-1;
-            }
-            shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+        if(!mktemp(file_mask)) {
+            apc_eprint("apc_mmap: mktemp on %s failed:", file_mask);
+            goto error;
+        }
+        fd = shm_open(file_mask, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
+        if(fd == -1) {
+            apc_eprint("apc_mmap: shm_open on %s failed:", file_mask);
+            goto error;
+        }
+        if (ftruncate(fd, size) < 0) {
+            close(fd);
             shm_unlink(file_mask);
-            close(fd);
+            apc_eprint("apc_mmap: ftruncate failed:");
+            goto error;
         }
+        shm_unlink(file_mask);
+    } else {
         /*
-         * Support anonymous mmap through the /dev/zero interface as well
-         */
-        else if(!strcmp(file_mask,"/dev/zero")) {
-            fd = open("/dev/zero", O_RDWR, S_IRUSR | S_IWUSR);
-            if(fd == -1) {
-                apc_eprint("apc_mmap: open on /dev/zero failed:");
-                return (void *)-1;
-            }
-            shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
-            close(fd);
-        }
-        /* 
          * Otherwise we do a normal filesystem mmap
          */
-        else {
-            fd = mkstemp(file_mask);
-            if(fd == -1) {
-                apc_eprint("apc_mmap: mkstemp on %s failed:", file_mask);
-                return (void *)-1;
-            }
-            if (ftruncate(fd, size) < 0) {
-                close(fd);
-                unlink(file_mask);
-                apc_eprint("apc_mmap: ftruncate failed:");
-            }
-            shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_NOSYNC, fd, 0);
+        fd = mkstemp(file_mask);
+        if(fd == -1) {
+            apc_eprint("apc_mmap: mkstemp on %s failed:", file_mask);
+            goto error;
+        }
+        if (ftruncate(fd, size) < 0) {
             close(fd);
             unlink(file_mask);
+            apc_eprint("apc_mmap: ftruncate failed:");
+            goto error;
         }
+        unlink(file_mask);
     }
-    if((long)shmaddr == -1) {
+
+    segment.shmaddr = (void *)mmap(NULL, size, PROT_READ | PROT_WRITE, flags, fd, 0);
+
+#ifdef APC_MEMPROTECT
+    if(remap) {
+        segment.roaddr = (void *)mmap(NULL, size, PROT_READ, flags, fd, 0);
+    } else {
+        segment.roaddr = NULL;
+    }
+#endif
+
+    if((long)segment.shmaddr == -1) {
         apc_eprint("apc_mmap: mmap failed:");
     }
-    return shmaddr;
+
+    if(fd != -1) close(fd);
+    
+    return segment;
+
+error:
+
+    segment.shmaddr = (void*)-1;
+#ifdef APC_MEMPROTECT
+    segment.roaddr = NULL;
+#endif
+    return segment;
 }
 
-void apc_unmap(void* shmaddr, size_t size)
+void apc_unmap(apc_segment_t *segment)
 {
-    if (munmap(shmaddr, size) < 0) {
+    if (munmap(segment->shmaddr, segment->size) < 0) {
         apc_wprint("apc_unmap: munmap failed:");
     }
+
+#ifdef APC_MEMPROTECT
+    if (segment->roaddr && munmap(segment->roaddr, segment->size) < 0) {
+        apc_wprint("apc_unmap: munmap failed:");
+    }
+#endif
+
 }
 
 #endif
@@ -134,6 +170,6 @@ void apc_unmap(void* shmaddr, size_t size)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
