@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_shm.c,v 3.11.2.2 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_shm.c 273457 2009-01-13 15:16:42Z gopalv $ */
 
 #include "apc_shm.h"
 #include "apc.h"
@@ -55,7 +55,7 @@ int apc_shm_create(const char* pathname, int proj, size_t size)
 
     key = IPC_PRIVATE;
 #ifndef PHP_WIN32
-	/* no ftok yet for win32 */
+    /* no ftok yet for win32 */
     if (pathname != NULL) {
         if ((key = ftok(pathname, proj)) < 0) {
             apc_eprint("apc_shm_create: ftok failed:");
@@ -77,27 +77,41 @@ void apc_shm_destroy(int shmid)
     shmctl(shmid, IPC_RMID, 0);
 }
 
-void* apc_shm_attach(int shmid)
+apc_segment_t apc_shm_attach(int shmid)
 {
-    void* shmaddr;  /* the shared memory address */
+    apc_segment_t segment; /* shm segment */
 
-    if ((long)(shmaddr = shmat(shmid, 0, 0)) == -1) {
+    if ((long)(segment.shmaddr = shmat(shmid, 0, 0)) == -1) {
         apc_eprint("apc_shm_attach: shmat failed:");
     }
 
+#ifdef APC_MEMPROTECT
+    
+    if ((long)(segment.roaddr = shmat(shmid, 0, SHM_RDONLY)) == -1) {
+        segment.roaddr = NULL;
+    }
+
+#endif
+
     /*
      * We set the shmid for removal immediately after attaching to it. The
      * segment won't disappear until all processes have detached from it.
      */
     apc_shm_destroy(shmid);
-    return shmaddr;
+    return segment;
 }
 
-void apc_shm_detach(void* shmaddr)
+void apc_shm_detach(apc_segment_t* segment)
 {
-    if (shmdt(shmaddr) < 0) {
+    if (shmdt(segment->shmaddr) < 0) {
         apc_eprint("apc_shm_detach: shmdt failed:");
     }
+
+#ifdef APC_MEMPROTECT
+    if (segment->roaddr && shmdt(segment->roaddr) < 0) {
+        apc_eprint("apc_shm_detach: shmdt failed:");
+    }
+#endif
 }
 
 /*
@@ -105,6 +119,6 @@ void apc_shm_detach(void* shmaddr)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
