@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_shm.h,v 3.8.2.1 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_shm.h 273456 2009-01-13 14:43:58Z gopalv $ */
 
 #ifndef APC_SHM_H
 #define APC_SHM_H
@@ -35,12 +35,14 @@
 #include <time.h>
 #endif
 
+#include "apc_sma.h"
+
 /* Wrapper functions for unix shared memory */
 
 extern int apc_shm_create(const char* name, int proj, size_t size);
 extern void apc_shm_destroy(int shmid);
-extern void* apc_shm_attach(int shmid);
-extern void apc_shm_detach(void* shmaddr);
+extern apc_segment_t apc_shm_attach(int shmid);
+extern void apc_shm_detach(apc_segment_t* segment);
 
 #endif
 
@@ -49,6 +51,6 @@ extern void apc_shm_detach(void* shmaddr);
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
