@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 
    This software was contributed to PHP by Facebook Inc. in 2007.
-   
+
    Future revisions and derivatives of this source code must acknowledge
    Facebook Inc. as the original contributor of this module by leaving
    this note intact in the source code.
@@ -24,7 +24,7 @@
    All other licensing and usage conditions are those of the PHP Group.
  */
 
- /* $Id: apc_signal.c,v 1.2.2.3 2008/05/11 18:57:00 rasmus Exp $ */
+ /* $Id: apc_signal.c 268255 2008-11-04 05:42:11Z rasmus $ */
 
  /* Allows apc to install signal handlers and maintain signalling
     to already registered handlers. Registers all signals that
@@ -34,6 +34,9 @@
     be called in request init (RINIT)
   */
 
+#include "apc.h"
+
+#if HAVE_SIGACTION
 #include <signal.h>
 #include "apc_globals.h"
 #include "apc_sma.h"
@@ -87,7 +90,6 @@ static void apc_rehandle_signal(int signo, siginfo_t *siginfo, void *context)
  */
 static int apc_register_signal(int signo, void (*handler)(int, siginfo_t*, void*))
 {
-#if HAVE_SIGACTION
     struct sigaction sa = {{0}};
     apc_signal_entry_t p_sig = {0};
 
@@ -100,7 +102,7 @@ static int apc_register_signal(int signo, void (*handler)(int, siginfo_t*, void*
             p_sig.signo = signo;
             p_sig.siginfo = ((sa.sa_flags & SA_SIGINFO) == SA_SIGINFO);
             p_sig.handler = (void *)sa.sa_handler;
-           
+
             apc_signal_info.prev = (apc_signal_entry_t **)apc_erealloc(apc_signal_info.prev, (apc_signal_info.installed+1)*sizeof(apc_signal_entry_t *));
             apc_signal_info.prev[apc_signal_info.installed] = (apc_signal_entry_t *)apc_emalloc(sizeof(apc_signal_entry_t));
             *apc_signal_info.prev[apc_signal_info.installed++] = p_sig;
@@ -123,7 +125,6 @@ static int apc_register_signal(int signo, void (*handler)(int, siginfo_t*, void*
 
         return SUCCESS;
     }
-#endif
     return FAILURE;
 } /* }}} */
 
@@ -182,11 +183,13 @@ void apc_shutdown_signals()
     }
 }
 
+#endif  /* HAVE_SIGACTION */
+
 /*
  * Local variables:
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
