@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_sma.c,v 1.69.2.2 2008/03/28 20:17:47 rasmus Exp $ */
+/* $Id: apc_sma.c 284580 2009-07-22 06:05:31Z kalle $ */
 
 #include "apc_sma.h"
 #include "apc.h"
@@ -34,65 +34,62 @@
 #include "apc_lock.h"
 #include "apc_shm.h"
 #include "apc_cache.h"
+
 #include <limits.h>
-#if APC_MMAP
-void *apc_mmap(char *file_mask, size_t size);
-void apc_unmap(void* shmaddr, size_t size);
-#endif
+#include "apc_mmap.h"
 
 #ifdef HAVE_VALGRIND_MEMCHECK_H
 #include <valgrind/memcheck.h>
 #endif
 
-/* {{{ locking macros */
-#define LOCK(c)         { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_lock(c); }
-#define RDLOCK(c)       { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_rdlock(c); }
-#define UNLOCK(c)       { apc_lck_unlock(c); HANDLE_UNBLOCK_INTERRUPTIONS(); }
-/* }}} */
-
 enum { DEFAULT_NUMSEG=1, DEFAULT_SEGSIZE=30*1024*1024 };
 
 static int sma_initialized = 0;     /* true if the sma has been initialized */
-static unsigned int sma_numseg;     /* number of shm segments to allow */
+static uint sma_numseg;     /* number of shm segments to allow */
 static size_t sma_segsize;          /* size of each shm segment */
-static size_t* sma_segments;        /* array of shm segment ids */
-static void** sma_shmaddrs;         /* array of shm segment addresses */
+static apc_segment_t* sma_segments; /* array of shm segments */
 static int sma_lastseg = 0;         /* index of MRU segment */
 
-typedef struct header_t header_t;
-struct header_t {
+typedef struct sma_header_t sma_header_t;
+struct sma_header_t {
     apc_lck_t sma_lock;     /* segment lock, MUST BE ALIGNED for futex locks */
     size_t segsize;         /* size of entire segment */
     size_t avail;           /* bytes available (not necessarily contiguous) */
-    size_t nfoffset;        /* start next fit search from this offset       */
 #if ALLOC_DISTRIBUTION
     size_t adist[30];
 #endif
 };
 
+#define SMA_HDR(i)  ((sma_header_t*)((sma_segments[i]).shmaddr))
+#define SMA_ADDR(i) ((char*)(SMA_HDR(i)))
+#define SMA_RO(i)   ((char*)(sma_segments[i]).roaddr)
+#define SMA_LCK(i)  ((SMA_HDR(i))->sma_lock)
+
 
 /* do not enable for threaded http servers */
 /* #define __APC_SMA_DEBUG__ 1 */
 
 #ifdef __APC_SMA_DEBUG__
-/* global counter for identifying blocks 
+/* global counter for identifying blocks
  * Technically it is possible to do the same
  * using offsets, but double allocations of the
  * same offset can happen. */
 static volatile size_t block_id = 0;
 #endif
 
-#define APC_SMA_CANARIES 1   
+#define APC_SMA_CANARIES 1
 
 typedef struct block_t block_t;
 struct block_t {
     size_t size;       /* size of this block */
-    size_t next;       /* offset in segment of next free block */
+    size_t prev_size;  /* size of sequentially previous block, 0 if prev is allocated */
+    size_t fnext;      /* offset in segment of next free block */
+    size_t fprev;      /* offset in segment of prev free block */
 #ifdef APC_SMA_CANARIES
     size_t canary;     /* canary to check for memory overwrites */
 #endif
 #ifdef __APC_SMA_DEBUG__
-    size_t id;         /* identifier for the memory block */ 
+    size_t id;         /* identifier for the memory block */
 #endif
 };
 
@@ -103,6 +100,10 @@ struct block_t {
 #define BLOCKAT(offset) ((block_t*)((char *)shmaddr + offset))
 #define OFFSET(block) ((size_t)(((char*)block) - (char*)shmaddr))
 
+/* macros for getting the next or previous sequential block */
+#define NEXT_SBLOCK(block) ((block_t*)((char*)block + block->size))
+#define PREV_SBLOCK(block) (block->prev_size ? ((block_t*)((char*)block - block->prev_size)) : NULL)
+
 /* Canary macros for setting, checking and resetting memory canaries */
 #ifdef APC_SMA_CANARIES
     #define SET_CANARY(v) (v)->canary = 0x42424242
@@ -119,16 +120,70 @@ struct block_t {
 #define MINBLOCKSIZE (ALIGNWORD(1) + ALIGNWORD(sizeof(block_t)))
 /* }}} */
 
-/* {{{ sma_allocate: tries to allocate size bytes in a segment */
-static size_t sma_allocate(void* shmaddr, size_t size)
+#if 0
+/* {{{ sma_debug_state(apc_sma_segment_t *segment, int canary_check, int verbose)
+ *        useful for debuging state of memory blocks and free list, and sanity checking
+ */
+static void sma_debug_state(void* shmaddr, int canary_check, int verbose) {
+    sma_header_t *header = (sma_header_t*)shmaddr;
+    block_t *cur = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
+    block_t *prv = NULL;
+    size_t avail;
+
+    /* Verify free list */
+    if (verbose) apc_wprint("Free List: ");
+    while(1) {
+        if (verbose) apc_wprint(" 0x%x[%d] (s%d)", cur, OFFSET(cur), cur->size);
+        if (canary_check) CHECK_CANARY(cur);
+        if (!cur->fnext) break;
+        cur = BLOCKAT(cur->fnext);
+        avail += cur->size;
+        if (prv == cur) {
+            apc_wprint("Circular list detected!");
+            assert(0);
+        }
+        if (prv && cur->fprev != OFFSET(prv)) {
+            apc_wprint("Previous pointer does not point to previous!");
+            assert(0);
+        }
+        prv = cur;
+    }
+    assert(avail == header->avail);
+
+    /* Verify each block */
+    if (verbose) apc_wprint("Block List: ");
+    cur = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
+    while(1) {
+        if(!cur->fnext) {
+            if (verbose) apc_wprint(" 0x%x[%d] (s%d) (u)", cur, OFFSET(cur), cur->size);
+        } else {
+            if (verbose) apc_wprint(" 0x%x[%d] (s%d) (f)", cur, OFFSET(cur), cur->size);
+        }
+        if (canary_check) CHECK_CANARY(cur);
+        if (!cur->size && !cur->fnext) break;
+        if (!cur->size) {
+            cur = BLOCKAT(OFFSET(cur) + ALIGNWORD(sizeof(block_t)));
+        } else {
+            cur = NEXT_SBLOCK(cur);
+        }
+        if (prv == cur) {
+            apc_wprint("Circular list detected!");
+            assert(0);
+        }
+        prv = cur;
+    }
+}
+/* }}} */
+#endif
+
+/* {{{ sma_allocate: tries to allocate at least size bytes in a segment */
+static size_t sma_allocate(sma_header_t* header, size_t size, size_t fragment, size_t *allocated)
 {
-    header_t* header;       /* header of shared memory segment */
+    void* shmaddr;          /* header of shared memory segment */
     block_t* prv;           /* block prior to working block */
     block_t* cur;           /* working block in list */
     block_t* prvnextfit;    /* block before next fit */
     size_t realsize;        /* actual size of block needed, including header */
-    size_t last_offset;     /* save the last search offset */
-    int wrapped=0;
     const size_t block_size = ALIGNWORD(sizeof(struct block_t));
 
     realsize = ALIGNWORD(size + block_size);
@@ -137,95 +192,77 @@ static size_t sma_allocate(void* shmaddr, size_t size)
      * First, insure that the segment contains at least realsize free bytes,
      * even if they are not contiguous.
      */
-    header = (header_t*) shmaddr;
+    shmaddr = header;
+
     if (header->avail < realsize) {
         return -1;
     }
 
     prvnextfit = 0;     /* initially null (no fit) */
-    last_offset = 0;
-
-    /* If we have a next fit offset, start searching from there */
-    if(header->nfoffset) {
-        prv = BLOCKAT(header->nfoffset);
-        /* if prv is the last block, jump to the beginning */
-        if(prv->next == 0) {
-            prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
-            wrapped = 1;
-        }
-    } else {    
-        prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
-    }
-   
+    prv = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
     CHECK_CANARY(prv);
 
-    while (prv->next != header->nfoffset) {
-        cur = BLOCKAT(prv->next);
+    while (prv->fnext != 0) {
+        cur = BLOCKAT(prv->fnext);
 #ifdef __APC_SMA_DEBUG__
         CHECK_CANARY(cur);
 #endif
-        /* If it can fit realiszie bytes in cur block, stop searching */
+        /* If it can fit realsize bytes in cur block, stop searching */
         if (cur->size >= realsize) {
             prvnextfit = prv;
             break;
         }
-        last_offset = prv->next;
         prv = cur;
-        if(wrapped && (prv->next >= header->nfoffset)) break;
-
-        /* Check to see if we need to wrap around and search from the top */
-        if(header->nfoffset && prv->next == 0) {
-            prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
-#ifdef __APC_SMA_DEBUG__
-            CHECK_CANARY(prv);
-#endif
-            last_offset = 0;
-            wrapped = 1;
-        } 
     }
 
     if (prvnextfit == 0) {
-        header->nfoffset = 0;
         return -1;
     }
 
     prv = prvnextfit;
-    cur = BLOCKAT(prv->next);
+    cur = BLOCKAT(prv->fnext);
 
     CHECK_CANARY(prv);
     CHECK_CANARY(cur);
 
-    if (cur->size == realsize || (cur->size > realsize && cur->size < (realsize + (MINBLOCKSIZE * 2)))) {
+    if (cur->size == realsize || (cur->size > realsize && cur->size < (realsize + (MINBLOCKSIZE + fragment)))) {
         /* cur is big enough for realsize, but too small to split - unlink it */
-        prv->next = cur->next;
-    }
-    else {
+        *(allocated) = cur->size - block_size;
+        prv->fnext = cur->fnext;
+        BLOCKAT(cur->fnext)->fprev = OFFSET(prv);
+        NEXT_SBLOCK(cur)->prev_size = 0;  /* block is alloc'd */
+    } else {
+        /* nextfit is too big; split it into two smaller blocks */
         block_t* nxt;      /* the new block (chopped part of cur) */
-        size_t nxtoffset;  /* offset of the block currently after cur */
         size_t oldsize;    /* size of cur before split */
 
-        /* nextfit is too big; split it into two smaller blocks */
-        nxtoffset = cur->next;
         oldsize = cur->size;
-        prv->next += realsize;  /* skip over newly allocated block */
-        cur->size = realsize;   /* Set the size of this new block */
-        nxt = BLOCKAT(prv->next);
-        nxt->next = nxtoffset;  /* Re-link the shortened block */
-        nxt->size = oldsize - realsize;  /* and fix the size */
+        cur->size = realsize;
+        *(allocated) = cur->size - block_size;
+        nxt = NEXT_SBLOCK(cur);
+        nxt->prev_size = 0;                       /* block is alloc'd */
+        nxt->size = oldsize - realsize;           /* and fix the size */
+        NEXT_SBLOCK(nxt)->prev_size = nxt->size;  /* adjust size */
         SET_CANARY(nxt);
+
+        /* replace cur with next in free list */
+        nxt->fnext = cur->fnext;
+        nxt->fprev = cur->fprev;
+        BLOCKAT(nxt->fnext)->fprev = OFFSET(nxt);
+        BLOCKAT(nxt->fprev)->fnext = OFFSET(nxt);
 #ifdef __APC_SMA_DEBUG__
         nxt->id = -1;
 #endif
     }
 
+    cur->fnext = 0;
+
     /* update the block header */
     header->avail -= cur->size;
 #if ALLOC_DISTRIBUTION
     header->adist[(int)(log(size)/log(2))]++;
 #endif
 
-    header->nfoffset = last_offset;
-
     SET_CANARY(cur);
 #ifdef __APC_SMA_DEBUG__
     cur->id = ++block_id;
@@ -239,7 +276,7 @@ static size_t sma_allocate(void* shmaddr, size_t size)
 /* {{{ sma_deallocate: deallocates the block at the given offset */
 static size_t sma_deallocate(void* shmaddr, size_t offset)
 {
-    header_t* header;   /* header of shared memory segment */
+    sma_header_t* header;   /* header of shared memory segment */
     block_t* cur;       /* the new block to insert */
     block_t* prv;       /* the block before cur */
     block_t* nxt;       /* the block after cur */
@@ -250,58 +287,45 @@ static size_t sma_deallocate(void* shmaddr, size_t offset)
 
     /* find position of new block in free list */
     cur = BLOCKAT(offset);
-    prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
-   
-    CHECK_CANARY(cur);
 
-#ifdef __APC_SMA_DEBUG__
-    CHECK_CANARY(prv);
-    fprintf(stderr, "free(%p, size=%d,id=%d)\n", cur, (int)(cur->size), cur->id);
-#endif
-    while (prv->next != 0 && prv->next < offset) {
-        prv = BLOCKAT(prv->next);
-#ifdef __APC_SMA_DEBUG__
-        CHECK_CANARY(prv);
-#endif
-    }
-    
-    CHECK_CANARY(prv);
-
-    /* insert new block after prv */
-    cur->next = prv->next;
-    prv->next = offset;
-
-#ifdef __APC_SMA_DEBUG__
-    CHECK_CANARY(cur);
-    cur->id = -1;
-#endif
-    
     /* update the block header */
-    header = (header_t*) shmaddr;
+    header = (sma_header_t*) shmaddr;
     header->avail += cur->size;
     size = cur->size;
 
-    if (((char *)prv) + prv->size == (char *) cur) {
+    if (cur->prev_size != 0) {
+        /* remove prv from list */
+        prv = PREV_SBLOCK(cur);
+        BLOCKAT(prv->fnext)->fprev = prv->fprev;
+        BLOCKAT(prv->fprev)->fnext = prv->fnext;
         /* cur and prv share an edge, combine them */
-        prv->size += cur->size;
-        prv->next = cur->next;
+        prv->size +=cur->size;
         RESET_CANARY(cur);
         cur = prv;
     }
 
-    nxt = BLOCKAT(cur->next);
-
-    if (((char *)cur) + cur->size == (char *) nxt) {
+    nxt = NEXT_SBLOCK(cur);
+    if (nxt->fnext != 0) {
+        assert(NEXT_SBLOCK(NEXT_SBLOCK(cur))->prev_size == nxt->size);
         /* cur and nxt shared an edge, combine them */
+        BLOCKAT(nxt->fnext)->fprev = nxt->fprev;
+        BLOCKAT(nxt->fprev)->fnext = nxt->fnext;
         cur->size += nxt->size;
-        cur->next = nxt->next;
 #ifdef __APC_SMA_DEBUG__
         CHECK_CANARY(nxt);
         nxt->id = -1; /* assert this or set it ? */
 #endif
         RESET_CANARY(nxt);
     }
-    header->nfoffset = 0;  /* Reset the next fit search marker */
+
+    NEXT_SBLOCK(cur)->prev_size = cur->size;
+
+    /* insert new block after prv */
+    prv = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
+    cur->fnext = prv->fnext;
+    prv->fnext = OFFSET(cur);
+    cur->fprev = OFFSET(prv);
+    BLOCKAT(cur->fnext)->fprev = OFFSET(cur);
 
     return size;
 }
@@ -311,7 +335,7 @@ static size_t sma_deallocate(void* shmaddr, size_t offset)
 
 void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
 {
-    int i;
+    uint i;
 
     if (sma_initialized) {
         return;
@@ -336,45 +360,57 @@ void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
 
     sma_segsize = segsize > 0 ? segsize : DEFAULT_SEGSIZE;
 
-    sma_segments = (size_t*) apc_emalloc(sma_numseg*sizeof(size_t));
-    sma_shmaddrs = (void**) apc_emalloc(sma_numseg*sizeof(void*));
-    
+    sma_segments = (apc_segment_t*) apc_emalloc(sma_numseg*sizeof(apc_segment_t));
+
     for (i = 0; i < sma_numseg; i++) {
-        header_t*   header;
+        sma_header_t*   header;
         block_t*    block;
         void*       shmaddr;
 
 #if APC_MMAP
-        sma_segments[i] = sma_segsize;
-        sma_shmaddrs[i] = apc_mmap(mmap_file_mask, sma_segsize);
+        sma_segments[i] = apc_mmap(mmap_file_mask, sma_segsize);
         if(sma_numseg != 1) memcpy(&mmap_file_mask[strlen(mmap_file_mask)-6], "XXXXXX", 6);
 #else
-        sma_segments[i] = apc_shm_create(NULL, i, sma_segsize);
-        sma_shmaddrs[i] = apc_shm_attach(sma_segments[i]);
+        sma_segments[i] = apc_shm_attach(apc_shm_create(NULL, i, sma_segsize));
 #endif
-        shmaddr = sma_shmaddrs[i];
-    
-        header = (header_t*) shmaddr;
+        
+        sma_segments[i].size = sma_segsize;
+
+        shmaddr = sma_segments[i].shmaddr;
+
+        header = (sma_header_t*) shmaddr;
         apc_lck_create(NULL, 0, 1, header->sma_lock);
         header->segsize = sma_segsize;
-        header->avail = sma_segsize - ALIGNWORD(sizeof(header_t)) - ALIGNWORD(sizeof(block_t));
-        header->nfoffset = 0;
+        header->avail = sma_segsize - ALIGNWORD(sizeof(sma_header_t)) - ALIGNWORD(sizeof(block_t)) - ALIGNWORD(sizeof(block_t));
 #if ALLOC_DISTRIBUTION
-       	{
+        {
            int j;
-           for(j=0; j<30; j++) header->adist[j] = 0; 
+           for(j=0; j<30; j++) header->adist[j] = 0;
         }
-#endif 
-        block = BLOCKAT(ALIGNWORD(sizeof(header_t)));
+#endif
+        block = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
         block->size = 0;
-        block->next = ALIGNWORD(sizeof(header_t)) + ALIGNWORD(sizeof(block_t));
+        block->fnext = ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
+        block->fprev = 0;
+        block->prev_size = 0;
         SET_CANARY(block);
 #ifdef __APC_SMA_DEBUG__
         block->id = -1;
 #endif
-        block = BLOCKAT(block->next);
-        block->size = header->avail;
-        block->next = 0;
+        block = BLOCKAT(block->fnext);
+        block->size = header->avail - ALIGNWORD(sizeof(block_t));
+        block->fnext = OFFSET(block) + block->size;
+        block->fprev = ALIGNWORD(sizeof(sma_header_t));
+        block->prev_size = 0;
+        SET_CANARY(block);
+#ifdef __APC_SMA_DEBUG__
+        block->id = -1;
+#endif
+        block = BLOCKAT(block->fnext);
+        block->size = 0;
+        block->fnext = 0;
+        block->fprev =  ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
+        block->prev_size = header->avail - ALIGNWORD(sizeof(block_t));
         SET_CANARY(block);
 #ifdef __APC_SMA_DEBUG__
         block->id = -1;
@@ -386,70 +422,93 @@ void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
 /* {{{ apc_sma_cleanup */
 void apc_sma_cleanup()
 {
-    int i;
+    uint i;
 
     assert(sma_initialized);
 
     for (i = 0; i < sma_numseg; i++) {
-        apc_lck_destroy(((header_t*)sma_shmaddrs[i])->sma_lock);
+        apc_lck_destroy(SMA_LCK(i));
 #if APC_MMAP
-        apc_unmap(sma_shmaddrs[i], sma_segments[i]);
+        apc_unmap(&sma_segments[i]);
 #else
-        apc_shm_detach(sma_shmaddrs[i]);
+        apc_shm_detach(&sma_segments[i]);
 #endif
     }
     sma_initialized = 0;
     apc_efree(sma_segments);
-    apc_efree(sma_shmaddrs);
 }
 /* }}} */
 
-/* {{{ apc_sma_malloc */
-void* apc_sma_malloc(size_t n)
+/* {{{ apc_sma_malloc_ex */
+void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated)
 {
     size_t off;
-    int i;
-    size_t *orig_mem_size_ptr;
+    uint i;
 
     TSRMLS_FETCH();
     assert(sma_initialized);
-    LOCK(((header_t*)sma_shmaddrs[sma_lastseg])->sma_lock);
+    LOCK(SMA_LCK(sma_lastseg));
+
+    off = sma_allocate(SMA_HDR(sma_lastseg), n, fragment, allocated);
+
+    if(off == -1 && APCG(current_cache)) { 
+        /* retry failed allocation after we expunge */
+        UNLOCK(SMA_LCK(sma_lastseg));
+        APCG(current_cache)->expunge_cb(APCG(current_cache), n);
+        LOCK(SMA_LCK(sma_lastseg));
+        off = sma_allocate(SMA_HDR(sma_lastseg), n, fragment, allocated);
+    }
 
-    off = sma_allocate(sma_shmaddrs[sma_lastseg], n);
     if (off != -1) {
-        void* p = (void *)(((char *)(sma_shmaddrs[sma_lastseg])) + off);
-        if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) += n; }
-        UNLOCK(((header_t*)sma_shmaddrs[sma_lastseg])->sma_lock);
+        void* p = (void *)(SMA_ADDR(sma_lastseg) + off);
+        UNLOCK(SMA_LCK(sma_lastseg));
 #ifdef VALGRIND_MALLOCLIKE_BLOCK
         VALGRIND_MALLOCLIKE_BLOCK(p, n, 0, 0);
 #endif
         return p;
     }
-    UNLOCK(((header_t*)sma_shmaddrs[sma_lastseg])->sma_lock);
+    
+    UNLOCK(SMA_LCK(sma_lastseg));
 
     for (i = 0; i < sma_numseg; i++) {
         if (i == sma_lastseg) {
             continue;
         }
-        LOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
-        off = sma_allocate(sma_shmaddrs[i], n);
+        LOCK(SMA_LCK(i));
+        off = sma_allocate(SMA_HDR(i), n, fragment, allocated);
+        if(off == -1 && APCG(current_cache)) { 
+            /* retry failed allocation after we expunge */
+            UNLOCK(SMA_LCK(i));
+            APCG(current_cache)->expunge_cb(APCG(current_cache), n);
+            LOCK(SMA_LCK(i));
+            off = sma_allocate(SMA_HDR(i), n, fragment, allocated);
+        }
         if (off != -1) {
-            void* p = (void *)(((char *)(sma_shmaddrs[i])) + off);
-            if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) += n; }
-            UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
+            void* p = (void *)(SMA_ADDR(i) + off);
+            UNLOCK(SMA_LCK(i));
             sma_lastseg = i;
 #ifdef VALGRIND_MALLOCLIKE_BLOCK
             VALGRIND_MALLOCLIKE_BLOCK(p, n, 0, 0);
 #endif
             return p;
         }
-        UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
+        UNLOCK(SMA_LCK(i));
     }
 
     return NULL;
 }
 /* }}} */
 
+/* {{{ apc_sma_malloc */
+void* apc_sma_malloc(size_t n)
+{
+    size_t allocated;
+    void *p = apc_sma_malloc_ex(n, MINBLOCKSIZE, &allocated);
+
+    return p;
+}
+/* }}} */
+
 /* {{{ apc_sma_realloc */
 void* apc_sma_realloc(void *p, size_t n)
 {
@@ -477,10 +536,9 @@ char* apc_sma_strdup(const char* s)
 /* {{{ apc_sma_free */
 void apc_sma_free(void* p)
 {
-    int i;
+    uint i;
     size_t offset;
     size_t d_size;
-    TSRMLS_FETCH();
 
     if (p == NULL) {
         return;
@@ -488,41 +546,105 @@ void apc_sma_free(void* p)
 
     assert(sma_initialized);
 
+    
     for (i = 0; i < sma_numseg; i++) {
-        LOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
-        offset = (size_t)((char *)p - (char *)(sma_shmaddrs[i]));
-        if (p >= sma_shmaddrs[i] && offset < sma_segsize) {
-            d_size = sma_deallocate(sma_shmaddrs[i], offset);
-            if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) -= d_size; }
-            UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
+        offset = (size_t)((char *)p - SMA_ADDR(i));
+        if (p >= (void*)SMA_ADDR(i) && offset < sma_segsize) {
+            LOCK(SMA_LCK(i));
+            d_size = sma_deallocate(SMA_HDR(i), offset);
+            UNLOCK(SMA_LCK(i));
 #ifdef VALGRIND_FREELIKE_BLOCK
             VALGRIND_FREELIKE_BLOCK(p, 0);
 #endif
             return;
         }
-        UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
     }
 
     apc_eprint("apc_sma_free: could not locate address %p", p);
 }
 /* }}} */
 
+#ifdef APC_MEMPROTECT
+/* {{{ */
+void* apc_sma_protect(void *p)
+{
+    int i;
+    size_t offset;
+
+    if (p == NULL) {
+        return NULL;
+    }
+
+    if(SMA_RO(sma_lastseg) == NULL) return p;
+
+    offset = (size_t)((char *)p - SMA_ADDR(sma_lastseg));
+
+    if(p >= (void*)SMA_ADDR(sma_lastseg) && offset < sma_segsize) {
+        return SMA_RO(sma_lastseg) + offset;
+    }
+
+    for (i = 0; i < sma_numseg; i++) {
+        offset = (size_t)((char *)p - SMA_ADDR(i));
+        if (p >= (void*)SMA_ADDR(i) && offset < sma_segsize) {
+            return SMA_RO(i) + offset;
+        }
+    }
+
+    return NULL;
+}
+/* }}} */
+
+/* {{{ */
+void* apc_sma_unprotect(void *p)
+{
+    int i;
+    size_t offset;
+
+    if (p == NULL) {
+        return NULL;
+    }
+
+    if(SMA_RO(sma_lastseg) == NULL) return p;
+
+    offset = (size_t)((char *)p - SMA_RO(sma_lastseg));
+
+    if(p >= (void*)SMA_RO(sma_lastseg) && offset < sma_segsize) {
+        return SMA_ADDR(sma_lastseg) + offset;
+    }
+
+    for (i = 0; i < sma_numseg; i++) {
+        offset = (size_t)((char *)p - SMA_RO(i));
+        if (p >= (void*)SMA_RO(i) && offset < sma_segsize) {
+            return SMA_ADDR(i) + offset;
+        }
+    }
+
+    return NULL;
+}
+/* }}} */
+#else
+/* {{{ */
+void* apc_sma_protect(void *p) { return p; }
+void* apc_sma_unprotect(void *p) { return p; }
+/* }}} */
+#endif
+
 /* {{{ apc_sma_info */
 apc_sma_info_t* apc_sma_info(zend_bool limited)
 {
     apc_sma_info_t* info;
     apc_sma_link_t** link;
-    int i;
-	char* shmaddr;
-	block_t* prv;
-	
+    uint i;
+    char* shmaddr;
+    block_t* prv;
+
     if (!sma_initialized) {
         return NULL;
     }
 
     info = (apc_sma_info_t*) apc_emalloc(sizeof(apc_sma_info_t));
     info->num_seg = sma_numseg;
-    info->seg_size = sma_segsize - ALIGNWORD(sizeof(header_t)) - ALIGNWORD(sizeof(block_t));
+    info->seg_size = sma_segsize - (ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t)) + ALIGNWORD(sizeof(block_t)));
 
     info->list = apc_emalloc(info->num_seg * sizeof(apc_sma_link_t*));
     for (i = 0; i < sma_numseg; i++) {
@@ -533,28 +655,34 @@ apc_sma_info_t* apc_sma_info(zend_bool limited)
 
     /* For each segment */
     for (i = 0; i < sma_numseg; i++) {
-        RDLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
-        shmaddr = sma_shmaddrs[i];
-        prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
+        RDLOCK(SMA_LCK(i));
+        shmaddr = SMA_ADDR(i);
+        prv = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
 
         link = &info->list[i];
 
         /* For each block in this segment */
-        while (prv->next != 0) {
-            block_t* cur = BLOCKAT(prv->next);
+        while (BLOCKAT(prv->fnext)->fnext != 0) {
+            block_t* cur = BLOCKAT(prv->fnext);
 #ifdef __APC_SMA_DEBUG__
             CHECK_CANARY(cur);
 #endif
 
             *link = apc_emalloc(sizeof(apc_sma_link_t));
             (*link)->size = cur->size;
-            (*link)->offset = prv->next;
+            (*link)->offset = prv->fnext;
             (*link)->next = NULL;
             link = &(*link)->next;
 
             prv = cur;
+
+#if ALLOC_DISTRIBUTION
+            sma_header_t* header = (sma_header_t*) segment->shmaddr;
+            memcpy(info->seginfo[i].adist, header->adist, sizeof(size_t) * 30);
+#endif
+
         }
-        UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
+        UNLOCK(SMA_LCK(i));
     }
 
     return info;
@@ -583,10 +711,10 @@ void apc_sma_free_info(apc_sma_info_t* info)
 size_t apc_sma_get_avail_mem()
 {
     size_t avail_mem = 0;
-    int i;
-    
+    uint i;
+
     for (i = 0; i < sma_numseg; i++) {
-        header_t* header = (header_t*) sma_shmaddrs[i];
+        sma_header_t* header = SMA_HDR(i);
         avail_mem += header->avail;
     }
     return avail_mem;
@@ -595,42 +723,16 @@ size_t apc_sma_get_avail_mem()
 
 #if ALLOC_DISTRIBUTION
 size_t *apc_sma_get_alloc_distribution(void) {
-    header_t* header = (header_t*) sma_shmaddrs[0];
-    return header->adist; 
+    sma_header_t* header = (sma_header_t*) segment->sma_shmaddr;
+    return header->adist;
 }
 #endif
 
-#if 0
-/* {{{ apc_sma_check_integrity */
-void apc_sma_check_integrity()
-{
-    int i;
-
-    /* For each segment */
-    for (i = 0; i < sma_numseg; i++) {
-        char* shmaddr = sma_shmaddrs[i];
-        header_t* header = (header_t*) shmaddr;
-        block_t* prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
-        int avail = 0;
-
-        /* For each block in this segment */
-        while (prv->next != 0) {
-            block_t* cur = BLOCKAT(prv->next);
-            avail += cur->size;
-            prv = cur;
-        }
-
-        assert(avail == header->avail);
-    }
-}
-/* }}} */
-#endif
-
 /*
  * Local variables:
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
