@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_sma.h,v 1.18.2.2 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_sma.h 277617 2009-03-23 02:16:24Z shire $ */
 
 #ifndef APC_SMA_H
 #define APC_SMA_H
@@ -36,9 +36,20 @@
 
 /* Simple shared memory allocator */
 
+typedef struct _apc_segment_t apc_segment_t;
+
+struct _apc_segment_t {
+    size_t size;
+    void* shmaddr;
+#ifdef APC_MEMPROTECT
+    void* roaddr;
+#endif
+};
+
 extern void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask);
 extern void apc_sma_cleanup();
 extern void* apc_sma_malloc(size_t size);
+extern void* apc_sma_malloc_ex(size_t size, size_t fragment, size_t* allocated);
 extern void* apc_sma_realloc(void* p, size_t size);
 extern char* apc_sma_strdup(const char *s);
 extern void apc_sma_free(void* p);
@@ -46,6 +57,9 @@ extern void apc_sma_free(void* p);
 extern size_t *apc_sma_get_alloc_distribution();
 #endif
 
+extern void* apc_sma_protect(void *p);
+extern void* apc_sma_unprotect(void *p);
+
 /* {{{ struct definition: apc_sma_link_t */
 typedef struct apc_sma_link_t apc_sma_link_t;
 struct apc_sma_link_t {
@@ -59,7 +73,7 @@ struct apc_sma_link_t {
 typedef struct apc_sma_info_t apc_sma_info_t;
 struct apc_sma_info_t {
     int num_seg;            /* number of shared memory segments */
-    long seg_size;           /* size of each shared memory segment */
+    size_t seg_size;           /* size of each shared memory segment */
     apc_sma_link_t** list;  /* there is one list per segment */
 };
 /* }}} */
@@ -83,6 +97,6 @@ typedef union { void* p; int i; long l; double d; void (*f)(); } apc_word_t;
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
